@@ -70,8 +70,10 @@ def _determine_maitai_host():
     if gcp_zone:
         if "us-west1" in gcp_zone:
             return "https://api.gcp.us-west1.trymaitai.ai"
-        elif "us-central1" in gcp_zone:
-            return "https://api.gcp.us-central1.trymaitai.ai"
+        # elif "us-central1" in gcp_zone:
+        #     return "https://api.gcp.us-central1.trymaitai.ai"
+        elif "us-south1" in gcp_zone:
+            return "https://api.gcp.us-south1.trymaitai.ai"
 
     azure_region = _get_azure_instance_metadata()
     if azure_region == "westus2":
@@ -141,7 +143,7 @@ class MaitaiClient:
         headers = {
             "Content-Type": "application/json",
             "x-api-key": api_key,
-            "x-client-version": version,
+            "User-Agent": version,
         }
         labels = {
             "cause": error,
@@ -160,7 +162,7 @@ class MaitaiClient:
         headers = {
             "Content-Type": "application/json",
             "x-api-key": api_key,
-            "x-client-version": version,
+            "User-Agent": version,
         }
         labels = {
             "cause": error,
@@ -178,7 +180,7 @@ class MaitaiClient:
         headers = {
             "Content-Type": "application/json",
             "x-api-key": api_key,
-            "x-client-version": version,
+            "User-Agent": version,
         }
 
         response = httpx.get(url, headers=headers, timeout=15)
