@@ -15,76 +15,41 @@ from typing import (
 
 import openai
 import openai.types.chat as openai_types
-from betterproto import Casing
+import pydantic
 from openai.types import ChatModel
-from pydantic.json_schema import GenerateJsonSchema
 
-import maitai_gen.chat as chat_types
+import maitai.models.chat as chat_types
+from maitai._pydantic import (
+    is_basemodel_type,
+    is_dataclass_like_type,
+    to_strict_json_schema,
+)
 from maitai._types import ToolFunction
 from maitai.tools import Tools
 from maitai_common.version import version
 
-__version__ = f"{version}_python_{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
+__version__ = f"Maitai/Python{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro} {version}"
 
 CallableT = TypeVar("CallableT", bound=Callable[..., Any])
 
 
-class OpenAISchemaGenerator(GenerateJsonSchema):
-    def generate(self, schema, mode="validation") -> dict:
-        json_schema = super().generate(schema, mode=mode)
+def type_to_response_format_param(response_format):
+    json_schema_type: type[pydantic.BaseModel] | pydantic.TypeAdapter[Any] | None = None
 
-        # Inline the referenced objects
-        def resolve_refs(obj, processed_refs=None):
-            if processed_refs is None:
-                processed_refs = set()
+    if is_basemodel_type(response_format):
+        name = response_format.__name__
+        json_schema_type = response_format
+    elif is_dataclass_like_type(response_format):
+        name = response_format.__name__
+        json_schema_type = pydantic.TypeAdapter(response_format)
+    else:
+        raise TypeError(f"Unsupported response_format type - {response_format}")
 
-            if isinstance(obj, dict):
-                if "$ref" in obj and obj["$ref"].startswith("#/$defs/"):
-                    ref_name = obj["$ref"].split("/")[-1]
-
-                    # If we've seen this ref before, return a simplified version
-                    if ref_name in processed_refs:
-                        return {"type": "object"}
-
-                    processed_refs.add(ref_name)
-                    return resolve_refs(
-                        json_schema.get("$defs", {}).get(ref_name, {}), processed_refs
-                    )
-
-                return {k: resolve_refs(v, processed_refs) for k, v in obj.items()}
-            elif isinstance(obj, list):
-                return [resolve_refs(item, processed_refs) for item in obj]
-            return obj
-
-        # Get the main schema from $defs if it exists
-        main_schema = None
-        if "allOf" in json_schema and json_schema["allOf"]:
-            ref = json_schema["allOf"][0].get("$ref")
-            if ref and ref.startswith("#/$defs/"):
-                ref_name = ref.split("/")[-1]
-                main_schema = json_schema["$defs"][ref_name]
-
-        properties = resolve_refs(
-            main_schema["properties"] if main_schema else json_schema["properties"]
-        )
-
-        return {
-            "name": (
-                main_schema.get("title", "") if main_schema else schema.get("title", "")
-            ).lower()
-            + "_schema",
-            "strict": True,
-            "schema": {
-                "type": "object",
-                "properties": properties,
-                "required": (
-                    main_schema.get("required", [])
-                    if main_schema
-                    else json_schema.get("required", [])
-                ),
-                "additionalProperties": False,
-            },
-        }
+    return {
+        "schema": to_strict_json_schema(json_schema_type),
+        "name": name,
+        "strict": True,
+    }
 
 
 def required_args(*variants: Sequence[str]) -> Callable[[CallableT], CallableT]:
@@ -169,13 +134,13 @@ def quote(string: str) -> str:
 def convert_openai_chat_completion(
     chat: openai_types.ChatCompletion,
 ) -> chat_types.ChatCompletionResponse:
-    return chat_types.ChatCompletionResponse().from_dict(chat.to_dict())
+    return chat_types.ChatCompletionResponse.model_validate(chat.model_dump())
 
 
 def convert_open_ai_chat_completion_chunk(
     chunk: openai_types.ChatCompletionChunk,
 ) -> chat_types.ChatCompletionChunk:
-    return chat_types.ChatCompletionChunk().from_dict(chunk.to_dict())
+    return chat_types.ChatCompletionChunk.model_validate(chunk.model_dump())
 
 
 def get_chat_completion_params(
@@ -211,6 +176,7 @@ def get_chat_completion_params(
     top_p: Union[Optional[float], openai.NotGiven] = openai.NOT_GIVEN,
     user: Union[str, openai.NotGiven] = openai.NOT_GIVEN,
     parallel_tool_calls: Union[bool, openai.NotGiven] = openai.NOT_GIVEN,
+    reasoning_effort: Optional[str] = openai.NOT_GIVEN,
     extra_headers: Optional[Dict[str, str]] = None,
 ) -> Dict[str, Any]:
     params = {
@@ -238,6 +204,7 @@ def get_chat_completion_params(
         "user": user,
         "tool_choice": tool_choice,
         "parallel_tool_calls": parallel_tool_calls,
+        "reasoning_effort": reasoning_effort,
         "extra_headers": extra_headers,
     }
 
@@ -251,13 +218,11 @@ def get_chat_completion_params(
             "include_usage": stream_options.get("include_usage")
         }
 
-    if response_format != openai.NOT_GIVEN:
+    if response_format != openai.NOT_GIVEN and response_format is not None:
         if hasattr(response_format, "model_json_schema"):
             response_format = {
                 "type": "json_schema",
-                "json_schema": response_format.model_json_schema(
-                    schema_generator=OpenAISchemaGenerator, mode="serialization"
-                ),
+                "json_schema": type_to_response_format_param(response_format),
             }
         params["response_format"] = {"type": response_format["type"]}
         if response_format.get("json_schema"):
@@ -266,11 +231,13 @@ def get_chat_completion_params(
     if tools != openai.NOT_GIVEN:
         params["tools"] = []
         if isinstance(tools, Tools):
-            params["tools"] = [tool.to_dict() for tool in tools.get_tool_definitions()]
+            params["tools"] = [
+                tool.model_dump() for tool in tools.get_tool_definitions()
+            ]
         elif tools:
             for tool in tools:
                 if hasattr(tool, "__tool__"):
-                    params["tools"].append(tool.__tool__.to_dict())
+                    params["tools"].append(tool.__tool__.model_dump())
                     continue
 
                 tool_dict = {"type": tool["type"]}
@@ -297,8 +264,8 @@ def get_chat_completion_params(
                         "type": function["parameters"].get("type", ""),
                         "properties": properties,
                         "required": function["parameters"].get("required", []),
-                        "additional_properties": function["parameters"].get(
-                            "additional_properties", False
+                        "additionalProperties": function["parameters"].get(
+                            "additionalProperties", False
                         ),
                         "enum": function["parameters"].get("enum", []),
                     }
@@ -318,8 +285,8 @@ def chat_completion_chunk_to_response(
     final_chunk: chat_types.ChatCompletionChunk,
     content: Union[str, List[chat_types.ChatMessage]],
 ):
-    chat_completion_response = chat_types.ChatCompletionResponse().from_dict(
-        final_chunk.to_dict(casing=Casing.SNAKE)
+    chat_completion_response = chat_types.ChatCompletionResponse.model_validate(
+        final_chunk.model_dump()
     )
     if isinstance(content, str):
         chat_completion_response.choices[0].message = chat_types.ChatMessage(
@@ -328,9 +295,9 @@ def chat_completion_chunk_to_response(
     if isinstance(content, List):
         chat_completion_response.choices[0].message = content[-1]
     if isinstance(chat_completion_response.choices[0].message, dict):
-        chat_completion_response.choices[
-            0
-        ].message = chat_types.ChatMessage().from_dict(
-            chat_completion_response.choices[0].message
+        chat_completion_response.choices[0].message = (
+            chat_types.ChatMessage.model_validate(
+                chat_completion_response.choices[0].message
+            )
         )
     return chat_completion_response
